function [cqes] = CQE(X, n1, n2 )
% Color image Quality Enhancement (CQE)
% 
% No reference quality measure for color image according to:
%
% K. Panetta, Ch. Gao, S. Agaian, "No Reference Color Image Contrast and
%           Qualitzy Measures," IEEE Tr. on Consumer Elect., Vol. 59, No.3
%           August 2013, pp.643-651.
% 
%   INPUTS:
%          X - RGB color image
%          k1, k2 - block size in each dimension such that: 3x3 up to 5x5
%
%  OUTPUT:
%      cqes - structure with four cells related respectively to CQE, colorfulness, sharpness and contrast
%
% implementation: I. Kopriva, IRB, January, 2014.
%

[I1 I2 I3] = size(X);

X = double(X);

lam = [0.299 0.587 0.114];
c = [0.4358 0.1722 0.3920];

% calculate colorfulness
alpha = X(:,:,1)-X(:,:,2);
beta = 0.5*(X(:,:,1)+X(:,:,2))-X(:,:,3);
vara = var(reshape(alpha,1,I1*I2));
mua = mean(reshape(alpha,1,I1*I2));
varb = var(reshape(beta,1,I1*I2));
mub = mean(reshape(beta,1,I1*I2));
colorfulness = 0.02*log(vara/power(abs(mua),0.2))*log(varb/power(abs(mub),0.2));

% calculate sharpness and contrast
for n=1:3
    XS(:,:,n)=edge(X(:,:,n),'sobel');
    XS(:,:,n)=X(:,:,n).*XS(:,:,n);
end

xm = max(max(max(X)));
srgb2lab = makecform('srgb2lab');
lab2srgb = makecform('lab2srgb');

X_lab = applycform(X/xm, srgb2lab); % convert to L*a*b*

% the values of luminosity can span a range from 0 to 100; scale them
% to [0 1] range (appropriate for MATLAB(R) intensity images of class double)
% before applying the three contrast enhancement techniques
max_luminosity = 100;
XL = X_lab(:,:,1)/max_luminosity;


% calculate number of blocks
k1 = I1-n1+1;
k2 = I2-n2+1;

EMEsh = zeros(3,1);
AMEc = 0;
for i=1:I1-n1+1
    for j=1:I2-n2+1
        Imaxkl=max(max(XL(i:i+n1-1,j:j+n2-1))) + 2e-16;
        Iminkl=min(min(XL(i:i+n1-1,j:j+n2-1))) + 1e-16;
        AMEc = AMEc + sqrt(log((Imaxkl+Iminkl)/(Imaxkl-Iminkl)))/k1/k2;
        for k=1:3
           Imaxkl=max(max(XS(i:i+n1-1,j:j+n2-1,k)))+1e-16;
           Iminkl=min(min(XS(i:i+n1-1,j:j+n2-1,k)))+1e-16;
           EMEsh(k)=EMEsh(k) + log(Imaxkl/Iminkl)*2/k1/k2;
         end
    end
end


contrast = AMEc;
sharpness = (lam(1)*EMEsh(1) + lam(2)*EMEsh(2) + lam(3)*EMEsh(3))/1;

cqe = c(1)*colorfulness + c(2)*sharpness + c(3)*contrast;

 % returned structure
cqes=struct('cqe',cqe,'colorfulness',colorfulness, 'sharpness', sharpness,'contrast',contrast);

end